/* -----------------------------------------------------------------------------
 * Copyright (c) 2013-2015 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        8. December 2015
 * $Revision:    V1.0
 *
 * Project:      CAN Driver Definitions for NXP LPC43xx
 * -------------------------------------------------------------------------- */

#ifndef __CAN_LPC43XX_H
#define __CAN_LPC43XX_H

#include <stdint.h>
#include <string.h>

#include "Driver_CAN.h"

#include "LPC43xx.h"
#include "SCU_LPC43xx.h"

#include "RTE_Device.h"
#include "RTE_Components.h"

#ifndef RTE_CAN_CAN0
#define RTE_CAN_CAN0                   (0U)
#endif
#ifndef RTE_CAN_CAN1
#define RTE_CAN_CAN1                   (0U)
#endif

#if   ((RTE_CAN_CAN0 == 0U) && (RTE_CAN_CAN1 == 0U))
#error "No CAN is enabled in the RTE_Device.h!"
#endif

#if    (RTE_CAN_CAN1 == 1U)
#define CAN_CTRL_NUM                   (2U)
#else
#define CAN_CTRL_NUM                   (1U)
#endif

#if    (defined(CORE_M0SUB))
#define MX_C_CAN0_IRQn                  M0S_C_CAN0_IRQn
#define MX_C_CAN0_IRQHandler            M0S_C_CAN0_IRQHandler
#define MX_C_CAN1_IRQn                  M0S_C_CAN1_IRQn
#define MX_C_CAN1_IRQHandler            M0S_USART2_OR_C_CAN1_IRQHandler
#elif  (defined(CORE_M0))
#define MX_C_CAN0_IRQn                  M0_C_CAN0_IRQn
#define MX_C_CAN0_IRQHandler            M0_C_CAN0_IRQHandler
#define MX_C_CAN1_IRQn                  M0_C_CAN1_IRQn
#define MX_C_CAN1_IRQHandler            M0_USART2_OR_C_CAN1_IRQHandler
#else
#define MX_C_CAN0_IRQn                  C_CAN0_IRQn
#define MX_C_CAN0_IRQHandler            C_CAN0_IRQHandler
#define MX_C_CAN1_IRQn                  C_CAN1_IRQn
#define MX_C_CAN1_IRQHandler            C_CAN1_IRQHandler
#endif

// CNTL register bit definitions
#define CNTL_INIT_Msk                  (1U      <<  0)
#define CNTL_IE_Msk                    (1U      <<  1)
#define CNTL_SIE_Msk                   (1U      <<  2)
#define CNTL_EIE_Msk                   (1U      <<  3)
#define CNTL_DAR_Msk                   (1U      <<  5)
#define CNTL_CCE_Msk                   (1U      <<  6)
#define CNTL_TEST_Msk                  (1U      <<  7)

// STAT register bit definitions
#define STAT_LEC_Msk                   (7U      <<  0)
#define STAT_TXOK_Msk                  (1U      <<  3)
#define STAT_RXOK_Msk                  (1U      <<  4)
#define STAT_EPASS_Msk                 (1U      <<  5)
#define STAT_EWARN_Msk                 (1U      <<  6)
#define STAT_BOFF_Msk                  (1U      <<  7)

// EC register bit definitions
#define EC_TEC_7_0_Msk                 (0xFFU   <<  0)
#define EC_REC_6_0_Msk                 (0x7FU   <<  8)
#define EC_RP_Msk                      (1U      << 15)

// BT register bit definitions
#define BT_BRP_Msk                     (0x3FU   <<  0)
#define BT_SJW_Msk                     (3U      <<  6)
#define BT_TSEG1_Msk                   (0x0FU   <<  8)
#define BT_TSEG2_Msk                   (0x07U   << 12)

// TEST register bit definitions
#define TEST_BASIC_Msk                 (1U      <<  2)
#define TEST_SILENT_Msk                (1U      <<  3)
#define TEST_LBACK_Msk                 (1U      <<  4)
#define TEST_TX1_0_Msk                 (3U      <<  5)
#define TEST_RX_Msk                    (1U      <<  7)

// BRPE register bit definitions
#define BRPE_BRPE_Msk                  (0x0FU   <<  0)

// CMDREQ register bit definitions
#define IF_CMDREQ_MESSABE_NUMBER_Msk   (0x3FU   <<  0)
#define IF_CMDREQ_BUSY_Msk             (1U      << 15)

// CMDMSK_W/R register bit definitions
#define IF_CMDMSK_DATA_B_Msk           (1U      <<  0)
#define IF_CMDMSK_DATA_A_Msk           (1U      <<  1)
#define IF_CMDMSK_TXRQST_Msk           (1U      <<  2)
#define IF_CMDMSK_NEWDAT_Msk           (1U      <<  2)
#define IF_CMDMSK_CLRINTPND_Msk        (1U      <<  3)
#define IF_CMDMSK_CTRL_Msk             (1U      <<  4)
#define IF_CMDMSK_ARB_Msk              (1U      <<  5)
#define IF_CMDMSK_MASK_Msk             (1U      <<  6)
#define IF_CMDMSK_WR_RD_Msk            (1U      <<  7)

// MSK1 register bit definitions
#define IF_MSK1_MSK15_0_Msk            (0xFFFFU <<  0)

// MSK2 register bit definitions
#define IF_MSK2_MSK28_16_Msk           (0x1FFFU <<  0)
#define IF_MSK2_MDIR_Msk               (1U      << 14)
#define IF_MSK2_MXTD_Msk               (1U      << 15)

// ARB1 register bit definitions
#define IF_ARB1_ID15_0_Msk             (0xFFFFU <<  0)

// ARB2 register bit definitions
#define IF_ARB2_ID28_16_Msk            (0x1FFFU <<  0)
#define IF_ARB2_DIR_Msk                (1U      << 13)
#define IF_ARB2_XTD_Msk                (1U      << 14)
#define IF_ARB2_MSGVAL_Msk             (1U      << 15)

// MCTRL register bit definitions
#define IF_MCTRL_DLC3_0_Msk            (0x0FU   <<  0)
#define IF_MCTRL_EOB_Msk               (1U      <<  7)
#define IF_MCTRL_TXRQST_Msk            (1U      <<  8)
#define IF_MCTRL_RMTEN_Msk             (1U      <<  9)
#define IF_MCTRL_RXIE_Msk              (1U      << 10)
#define IF_MCTRL_TXIE_Msk              (1U      << 11)
#define IF_MCTRL_UMASK_Msk             (1U      << 12)
#define IF_MCTRL_INTPND_Msk            (1U      << 13)
#define IF_MCTRL_MSGLST_Msk            (1U      << 14)
#define IF_MCTRL_NEWDAT_Msk            (1U      << 15)

#endif // __CAN_LPC43XX_H
